.include "tn2313def.inc"

;***** Konstanten definitions
.equ	F_CPU		= 4000000		;Quarzfrequenz
.equ	OVF0load	=-156			;timer0 reload value, prescaler=256 -> 156,25 Takte=10ms
.equ	OVF1load	=-15625			;timer1 reload value, prescaler=256 -> 15625 Takte=1s
.equ	TCCR0init	=1<<CS02		;Clock/256, start timer
.equ	TCCR1init	=1<<CS12		;Clock/256, start timer

;***** Pin definitions
.equ	RxD			=0				;Receive pin is PD0
.equ	TxD			=1				;Transmit pin is PD1
.equ	triac		=6				;Triac pin is PD6

;***** Status Byte definitions
.equ	delon		=0				;Bit 0 = Zeitverzgert ein luft
.equ	deloff		=1				;Bit 1 = Zeitverzgert aus luft
.equ	single		=2				;Bit 2 = einmaliger Zyklus
.equ	cycle		=3				;Bit 3 = dauerhafter Zyklus
.equ	dim			=4				;Bit 4 = DIMMEN
.equ	activeon	=5				;Bit 5 = Zeitbasis EIN aktiv
.equ	firstsingle	=6				;Bit 6 = Erster Teil einmaliger Zyklus aktiv
.equ	on			=7				;Bit 7 = ON

;***** Baud Rate definitions
.equ	doublebaud	= 1

.equ	baud300		= (F_CPU /    4800) - 1
.equ	baud600		= (F_CPU /    9600) - 1
.equ	baud1200	= (F_CPU /   19200) - 1
.equ	baud2400	= (F_CPU /   38400) - 1
.equ	baud4800	= (F_CPU /   76800) - 1
.equ	baud9600	= (F_CPU /  153600) - 1
.equ	baud14400	= (F_CPU /  230400) - 1
.equ	baud19200	= (F_CPU /  307200) - 1
.equ	baud28800	= (F_CPU /  460800) - 1
.equ	baud38400	= (F_CPU /  614400) - 1
.equ	baud57600	= (F_CPU /  921600) - 1
.equ	baud76800	= (F_CPU / 1228800) - 1
.equ	baud115200	= (F_CPU / 1843200) - 1
.equ	baud230400	= (F_CPU / 3686400) - 1

.equ	baud		= baud9600

;***** Global register variables
.equ	ontmrstart	=0
.def	ontmr1		=R0				;3 Byte Timer fuer ON-Zyklus
.def	ontmr2		=R1				;3 Byte Timer fuer ON-Zyklus
.def	ontmr3		=R2				;3 Byte Timer fuer ON-Zyklus
.equ	offtmrstart	=3
.def	offtmr1		=R3				;3 Byte Timer fuer OFF-Zyklus
.def	offtmr2		=R4				;3 Byte Timer fuer OFF-Zyklus
.def	offtmr3		=R5				;3 Byte Timer fuer OFF-Zyklus
.equ	runtmrstart	=6
.def	runtmr1		=R6				;3 Byte Timer fuer RUN-Zyklus
.def	runtmr2		=R7				;3 Byte Timer fuer RUN-Zyklus
.def	runtmr3		=R8				;3 Byte Timer fuer RUN-Zyklus

.equ	serstart	=9
.def	serbuf0		=R9				;8 Byte serieller Buffer
.def	serbuf1		=R10			;8 Byte serieller Buffer
.def	serbuf2		=R11			;8 Byte serieller Buffer
.def	serbuf3		=R12			;8 Byte serieller Buffer
.def	serbuf4		=R13			;8 Byte serieller Buffer
.def	serbuf5		=R14			;8 Byte serieller Buffer
.def	serbuf6		=R15			;8 Byte serieller Buffer
.def	serbuf7		=R16			;8 Byte serieller Buffer
.equ	serend		=17

.def	inttemp0	=R17			;temporary storage register fr interrupts
.def	inttemp1	=R18			;temporary storage register fr interrupts
.def	inttemp2	=R19			;temporary storage register fr interrupts
.def	temp0		=R20			;temporary storage register
.def	temp1		=R21			;temporary storage register
.def	temp2		=R22			;temporary storage register
.def	serbyte		=R23			;Data to be transmitted/received
.def	delay		=R24			;delay counter
.def	counter1	=R25			;counter fr interrupts
.def	counter2	=R26			;counter fr interrupts
.def	ignition	=R27			;counter fr zndung
.def	status		=R28			;Status der Steckdose, Bit0=dauer-Ein, Bit1=dimmen
.def	loopcnt		=R29			;loop-counter


.cseg
.org		0
		rjmp	RESET				;Reset Handler
		rjmp	EXT_INT0			;IRQ0 Handler
		rjmp	EXT_INT1			;IRQ1 Handler
		rjmp	TIM_CAPT1			;Timer1 Capture Handler
		rjmp	TIM_COMP1			;Timer1 Compare Handler
		rjmp	TIM_OVF1			;Timer1 Overflow Handler
		rjmp	TIM_OVF0			;Timer0 Overflow Handler
		rjmp	UART_RXC			;UART RX Complete Handler
		rjmp	UART_DRE			;UDR Empty Handler
		rjmp	UART_TXC			;UART TX Complete Handler
		rjmp	ANA_COMP			;Analog Comparator Handler

EXT_INT0:
EXT_INT1:
TIM_CAPT1:
TIM_COMP1:
UART_RXC:
UART_DRE:
UART_TXC:
		reti


;***** handle analog comperator interrupt
ANA_COMP:
		sbrc	status,dim			;Dimmen?
		rjmp	dimsetup			;ja, timer setzen
		sbrc	status,on			;ein?
		cbi		PORTD,triac			;znden EIN
		sbrs	status,on			;aus?
		sbi		PORTD,triac			;znden AUS
		reti
dimsetup:
		in		inttemp0,SREG		;SREG sichern
		ldi		inttemp1,OVF0load	;OVFload Clocks laden
		mov		inttemp2,ignition	;Zuendzeitpunkt addieren, darf nicht groesser als OVF0load-1 sein
		add		inttemp1,inttemp2
		out		TCNT0,inttemp1		;Clocks speichern
		ldi		inttemp1,TCCR0init
		out		TCCR0,inttemp1		;start Timer
ANA_CMPexit:
		out		SREG,inttemp0		;SREG wiederherstellen
		reti

;***** handle overflow 0 interrupt (dimmen)
TIM_OVF0:
		in		inttemp0,SREG		;SREG sichern
		ldi		inttemp1,0
		out		TCCR0,inttemp1		;stop Timer
		sbrc	status,on			;dimmen oder ein?
		cbi		PORTD,triac			;zuenden EIN
igndel:	dec		inttemp1			;256 Zyklen Warteschleife
		brne	igndel
		sbi		PORTD,triac			;zuenden AUS
		out		SREG,inttemp0		;SREG wiederherstellen
		reti

;***** handle overflow 1 interrupt (sekundentimer)
TIM_OVF1:
		in		inttemp0,SREG		;SREG sichern
		ldi		inttemp1,high(OVF1load)	;OVFload Clocks laden
		out		TCNT1H,inttemp1
		ldi		inttemp1,low(OVF1load)	;OVFload Clocks laden
		out		TCNT1L,inttemp1
		mov		inttemp1,status		;fr Test laden
		andi	inttemp1,0x0F		;Bits fr zyklische Vorgnge
		breq	OVF1exit			;nicht vorhanden? Ende
		ldi		inttemp1,1			;fr sub laden
		sub		runtmr3,inttemp1	;eine Sekunde abziehen
		ldi		inttemp1,0			;fr sub laden
		sbc		runtmr2,inttemp1
		sbc		runtmr1,inttemp1
		or		inttemp1,runtmr3	;auf 0 prfen
		or		inttemp1,runtmr2	;auf 0 prfen
		or		inttemp1,runtmr1	;auf 0 prfen
		brne	OVF1exit			;nicht 0, Ende
		;*** Delayed ON und OFF
		sbrc	status,delon
		ori		status,0x80			;einschalten
		sbrc	status,delon
		andi	status,0x90			;Zeitverzgert-ON abgearbeitet, Zyklusbits lschen
		sbrc	status,deloff
		andi	status,0x10			;Zeitverzgert-OFF abgearbeitet, lschen, ausschalten
		;*** Zyklus
		mov		inttemp1,status		;Zyklus aktiv?
		andi	inttemp1,0x0C		;Bits 2 und 3
		breq	OVF1exit			;nein, Ende
		bst		status,activeon		;activeon -> SREG.T
		bld		status,on			;neuer EIN-Zustand
		ldi		inttemp1,0xA0		;Maske zum Invertieren
		eor		status,inttemp1		;running-Zyklus setzen
		bst		status,single		;Single-Zyklus aktiv?
		brtc	reload				;nein, Dauerzyklus aktiv -> reload
		bst		status,firstsingle	;Erster Single-Zyklus aktiv?
		brts	reload				;ja, einmaliger reload
		andi	status,0x90			;Single-Zyklus komplett abgearbeitet, lschen
		rjmp	OVF1exit
reload:
		andi	status,0xBF			;First Single-Zyklus lschen
		push	ZH					;Register sichern
		push	ZL
		ldi		ZH,0				;RAM Adresse high immer 0
		ldi		ZL,ontmrstart		;Adresse ON-Timer
		sbrs	status,activeon
		ldi		ZL,offtmrstart		;Adresse OFF-Timer
		ld		inttemp1,Z+			;erstes Byte laden
		mov		runtmr1,inttemp1	;und speichern
		ld		inttemp1,Z+			;erstes Byte laden
		mov		runtmr2,inttemp1	;und speichern
		ld		inttemp1,Z			;erstes Byte laden
		mov		runtmr3,inttemp1	;und speichern
		pop		ZL
		pop		ZH					;Register wiederherstellen
OVF1exit:
		out		SREG,inttemp0		;SREG wiederherstellen
		reti

;***** Program Execution Starts Here
RESET:	;*** stack pointer setup
		ldi		temp0,RAMEND		;in proctype.def definiert
		out		SPL,temp0			;Stack Pointer laden
		;*** port setup
		ldi		temp0,0x40			;PD6 (triac) auf eins (pullup)
		out		PORTD,temp0
		ldi		temp0,0x40			;PD6 Ausgang (triac)
		out		DDRD,temp0
		;*** timer1 setup
		ldi		temp0,high(OVF1load);OVF1load Clocks laden
		out		TCNT1H,temp0
		ldi		temp0,low(OVF1load)	;OVF1load Clocks laden
		out		TCNT1L,temp0
		ldi		temp0,TCCR1init		;Clock/256, start timer
		out		TCCR1B,temp0
		;*** timer interrupts enable
		ldi		temp0,(1<<TOIE0) | (1<<TOIE1)	;Timer Overflow Interrupt enable
		out		TIMSK,temp0
		;*** UART setup
		ldi		temp0,baud			;baudrate divider
		out		UBRR,temp0
		ldi		temp0,(1<<RXEN) | (1<<TXEN)
  		out		UCR,temp0
		;*** analog comperator setup
;		sbi		ACSR,ACIS0			;analog comperator Interrupt on rising edge
;		sbi		ACSR,ACIS1
		sbi		ACSR,ACIE			;analog comperator Interrupt enable
		sbi		PORTB,0				;AIN0, pullup
		;*** Watchdog timer setup
		ldi		temp0,0x0C			;Watchdog Timer enable, Reset-Time 0,24s/256k cycles
		out		WDTCR,temp0
		;*** Variablen initialisieren
		ldi		ZL,0				;erste Variablenadresse laden
		ldi		ZH,0				;erste Variablenadresse laden
		clr		temp0				;0 laden
rstfill:
		st		Z+,temp0			;und speichern
		cpi		ZL,29				;Ende?
		brne	rstfill				;nein, weiter fllen
		sei							;global Interrupt enable
		ldi		ignition,ignoff
		ldi		status,0x80
		rjmp	main				;Beginn Hauptprogramm

mainerrtmp1:
		sei							;global Interrupt enable
		rjmp	mainerr				;Zwischenstop

;***** Kommandos, die nicht mit br erreicht werden knnen
cmde:
		cpi		ZL,serstart+4		;mehr oder weniger als vier Byte empfangen?
		brne	mainerrtmp1			;ja, error
		mov		temp1,serbuf2		;High Digit laden
		mov		temp2,serbuf3		;Low Digit laden
		rcall	hex2bin				;umrechnen
		ldi		ZH,high(2*dimtable)	;Adresse laden
		ldi		ZL,low(2*dimtable)
		add		ZL,temp0			;dim Wert laden
		ldi		temp1,0				;fuer adc
		adc		ZH,temp1			;Uebertrag addieren
		add		ZL,temp0			;dim Wert laden
		adc		ZH,temp1			;Uebertrag addieren
		mov		temp0,R0			;R0 sichern wegen LPM
		lpm							;dim Wert aus Tabelle laden
		mov		ignition,R0			;errechneten Winkel einstellen
		mov		R0,temp0			;R0 wiederherstellen
		mov		temp0,status		;fr Vergleich laden
		andi	temp0,0x4F			;Zyklus oder single aktiv?
		breq	cmde00				;nein, selbst einschalten + dimmen
		ori		status,0x10			;dimmen, on/off unbeeinflusst
		rjmp	mainok
cmde00:
		ldi		status,0x90			;dimmen + on
		rjmp	mainok
cmdrvh:
		cpi		ZL,serstart+1		;mehr oder weniger als ein Byte empfangen?
		brne	mainerrtmp			;ja, error
		cpi		temp0,'r'			;Status BITS ausgeben?
		brne	cmdrvh00
		ldi		serbyte,'0'			;fr Ausgabe laden, erstes Byte ist immer 0
		rcall	putchar				;ausgeben
		ldi		serbyte,'0'			;Als Ausgangswert laden
		sbrc	status,on			;ON Bit verarbeiten
		ldi		serbyte,'1'			;Ausgabebit setzen
		rcall	putchar				;ausgeben
		rjmp	mainok
cmdrvh00:
		cpi		temp0,'v'			;Versionsnummer ausgeben?
		brne	cmdrvh01
		ldi		serbyte,'0'			;fr Ausgabe laden
		rcall	putchar				;ausgeben
		ldi		serbyte,'.'			;fr Ausgabe laden
		rcall	putchar				;ausgeben
		ldi		serbyte,'1'			;fr Ausgabe laden
		rcall	putchar				;ausgeben
		rjmp	mainok
cmdrvh01:
		ldi		serbyte,':'			;fr Ausgabe laden
		rcall	putchar				;ausgeben
		ldi		serbyte,'('			;fr Ausgabe laden
		rcall	putchar				;ausgeben
		rjmp	mainok
cmdfg:
		cpi		ZL,serstart+8		;mehr oder weniger als acht Byte empfangen?
		brne	mainerrtmp			;ja, error
		cli							;global Interrupt disable
		ldi		ZL,ontmrstart		;Startadresse EIN Zeitbasis
		cpi		temp0,'g'			;soll AUS Basis gesetzt werden?
		brne	cmdfg00
		ldi		ZL,offtmrstart		;Startadresse AUS Zeitbasis
cmdfg00:
		mov		temp1,serbuf2		;fr Umrechnung laden
		mov		temp2,serbuf3		;fr Umrechnung laden
		rcall	hex2bin				;HEX temp1:temp2 -> temp
		st		Z,temp0				;speichern
		inc		ZL					;nchste Adresse
		mov		temp1,serbuf4		;fr Umrechnung laden
		mov		temp2,serbuf5		;fr Umrechnung laden
		rcall	hex2bin				;HEX temp1:temp2 -> temp
		st		Z,temp0				;speichern
		inc		ZL					;nchste Adresse
		mov		temp1,serbuf6		;fr Umrechnung laden
		mov		temp2,serbuf7		;fr Umrechnung laden
		rcall	hex2bin				;HEX temp1:temp2 -> temp
		st		Z,temp0				;speichern
		sei							;global Interrupt enable
		rjmp	mainok

mainerrtmp:
		sei							;global Interrupt enable
		rjmp	mainerr				;Zwischenstop

;***** Haupt-Programmschleife
main:
		ldi		ZL,serstart		;Startadresse serieller Puffer laden
		ldi		serbyte,13			;Enter
		rcall	putchar				;ausgeben
		ldi		serbyte,10			;Linefeed
		rcall	putchar				;ausgeben
		ldi		serbyte,'#'			;Prompt
		rcall	putchar				;ausgeben
mainlp:
		rcall	getchar				;auf Eingabe warten
		cpi		serbyte,10			;LF senden einige Terminals, ignorieren
		breq	mainlp				;weiter warten
		cpi		serbyte,13			;Enter?
		breq	maincmd				;ENTER empfangen, Kommando ausfhren
		cpi		serbyte,8			;Backspace?
		brne	bufftst				;Zeichen speichern, Echo und Ende prfen
		cpi		ZL,serstart		;sind Zeichen im Buffer?
		breq	mainlp				;nein, weiter warten
		rcall	putchar				;echo Backspace
		dec		ZL					;vorherige Adresse fr sbuffer-Pointer
		rjmp	mainlp				;weiter warten
bufftst:
		mov		temp0,serbyte		;Byte sichern, da putchar Inhalt zerstrt
		rcall	putchar				;echo
		cpi		ZL,serend			;Ende erreicht? OHNE Enter?
		breq	mainerrtmp			;ja, wieder von vorn
		st		Z,temp0				;sonst im Puffer speichern
		inc		ZL					;nchste Adresse fr sbuffer-Pointer
		rjmp	mainlp				;nein, nchstes Zeichen abholen
		;*** Als erstes zu Ein Byte-Kommandos verzweigen
maincmd:
		cpi		ZL,serstart		;sind Zeichen im Buffer?
		breq	main				;nein, neuer Prompt
		mov		temp0,serbuf0		;fr cpi laden
		ori		temp0,0x20			;Kommando in Kleinbuchstaben verwandeln
		mov		serbuf0,temp0		;fr spter wieder speichern
		cpi		temp0,'r'			;Status BYTE?
		breq	cmdrvhtmp
		cpi		temp0,'v'			;Versionsnummer?
		breq	cmdrvhtmp
		cpi		temp0,'h'			;Hilfe-Bildschirm?
		breq	cmdrvhtmp
		;*** Jetzt adressierte Steckdose prfen
		mov		temp0,serbuf1		;fr cpi laden
		cpi		temp0,'0'			;nur 0 und 1 sind mglich!
		breq	mainchk
		cpi		temp0,'1'			;nur 0 und 1 sind mglich!
		brne	mainerrtmp
mainchk:
		mov		temp0,serbuf0		;fr cpi laden
		cpi		temp0,'f'			;Zeitbasis EIN setzen?
		breq	cmdfgtmp
		cpi		temp0,'g'			;Zeitbasis EIN setzen?
		breq	cmdfgtmp
		cpi		temp0,'e'			;dimmen?
		breq	cmdetmp
		rjmp	cmdother

cmdetmp:
		rjmp	cmde
cmdfgtmp:
		rjmp	cmdfg
cmdrvhtmp:
		rjmp	cmdrvh

;***** running Timer laden, Z hat Adresse von Quelltimer, prfen auf 0!!
ldtmr:
		ld		temp0,Z				;erstes Byte laden
		mov		runtmr1,temp0		;und speichern
		mov		temp1,temp0			;fr 0 Test
		inc		ZL					;nchste Adresse
		ld		temp0,Z				;zweites Byte laden
		mov		runtmr2,temp0		;und speichern
		or		temp1,temp0			;fr 0 Test
		inc		ZL					;nchste Adresse
		ld		temp0,Z				;drittes Byte laden
		mov		runtmr3,temp0		;und speichern
		or		temp1,temp0			;fr 0 Test
		ldi		counter2,0			;damit die erste Sekunde voll ist
		ret

;***** Timer prfen, Z hat Adresse von Quelltimer, prfen auf 0!!
chktmr:
		ld		temp0,Z+			;erstes Byte laden
		mov		temp1,temp0			;fr 0 Test
		ld		temp0,Z+			;zweites Byte laden
		or		temp1,temp0			;fr 0 Test
		ld		temp0,Z				;drittes Byte laden
		or		temp1,temp0			;fr 0 Test
		ret

mainerrtmp2:
		sei							;global Interrupt enable
		rjmp	mainerr				;Zwischenstop

;***** Hier kann die Kommandolnge geprft werden
cmdother:
		cpi		ZL,serstart+2		;mehr oder weniger als zwei Byte empfangen?
		brne	mainerrtmp2			;ja, error
		cpi		temp0,'a'			;ausschalten?
		brne	cmdb
		ldi		status,0			;ausschalten
		rjmp	mainok
cmdb:
		cpi		temp0,'b'			;verzgert ausschalten?
		brne	cmdc
		cli							;global Interrupt disable
		ldi		ZL,ontmrstart		;Basisadresse bergeben
		rcall	ldtmr				;runtmr mit on-wert laden
		breq	mainerrtmp2			;running-timer=0 -> error
		andi	status,0x90			;alle Zyklus-Bits aus
		ori		status,0x22			;running-ON, Zeitverzgert off
		sei							;global Interrupt enable
		rjmp	mainok
cmdc:
		cpi		temp0,'c'			;einschalten?
		brne	cmdd
		ldi		status,0x80			;einschalten
		rjmp	mainok
cmdd:
		cpi		temp0,'d'			;verzgert einschalten?
		brne	cmdi
		cli							;global Interrupt disable
		ldi		ZL,offtmrstart		;Basisadresse bergeben
		rcall	ldtmr				;runtmr mit off-wert laden
		breq	mainerrtmp2			;running-timer=0 -> error
		andi	status,0x90			;alle Zyklus-Bits aus
		ori		status,0x01			;running-OFF, Zeitverzgert on
		sei							;global Interrupt enable
		rjmp	mainok
cmdi:
		cpi		temp0,'i'			;dimmen+?
		brne	cmdj
		dec		ignition			;dimmen+
		mov		temp0,ignition		;fr Ausgabe laden
		neg 	temp0				;und negieren
		rcall	bin2hex				;temp -> HEX temp1:temp2
		mov		serbyte,temp1		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		mov		serbyte,temp2		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		rjmp	mainok
cmdj:
		cpi		temp0,'j'			;dimmen-?
		brne	cmdk
		inc		ignition			;dimmen-
		mov		temp0,ignition		;fr Ausgabe laden
		neg 	temp0				;und negieren
		rcall	bin2hex				;temp -> HEX temp1:temp2
		mov		serbyte,temp1		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		mov		serbyte,temp2		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		rjmp	mainok
cmdk:
		cpi		temp0,'k'			;Zeitbasis EIN lesen?
		brne	cmdl
		ldi		ZL,ontmrstart		;Startadresse
ontmrloop:
		ld		temp0,Z				;erstes Byte laden
		inc		ZL					;nchste Adresse
		rcall	bin2hex				;temp -> HEX temp1:temp2
		mov		serbyte,temp1		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		mov		serbyte,temp2		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		cpi		ZL,ontmrstart+3	;Ende?
		brne	ontmrloop			;nein, weiter
		rjmp	mainok
cmdl:
		cpi		temp0,'l'			;Zeitbasis AUS lesen?
		brne	cmds
		ldi		ZL,offtmrstart		;Startadresse
offtmrloop:
		ld		temp0,Z				;erstes Byte laden
		inc		ZL					;nchste Adresse
		rcall	bin2hex				;temp -> HEX temp1:temp2
		mov		serbyte,temp1		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		mov		serbyte,temp2		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		cpi		ZL,offtmrstart+3	;Ende?
		brne	offtmrloop			;nein, weiter
		rjmp	mainok
cmds:
		cpi		temp0,'s'			;Status lesen?
		brne	cmdw
		mov		temp0,status		;Status laden
		rcall	bin2hex				;temp -> HEX temp1:temp2
		mov		serbyte,temp1		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		mov		serbyte,temp2		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		ldi		ZL,runtmrstart		;Startadresse
runtmrloop:
		ld		temp0,Z				;erstes Byte laden
		inc		ZL					;nchste Adresse
		rcall	bin2hex				;temp -> HEX temp1:temp2
		mov		serbyte,temp1		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		mov		serbyte,temp2		;fr Ausgabe laden
		rcall	putchar				;ausgeben
		cpi		ZL,runtmrstart+3	;Ende?
		brne	runtmrloop			;nein, weiter
		rjmp	mainok
cmdw:
		cpi		temp0,'w'			;Single OFF - ON?
		brne	cmdx
		cli							;global Interrupt disable
		ldi		ZL,offtmrstart		;Basisadresse bergeben
		rcall	chktmr				;runtmr mit on-wert laden
		breq	mainerr				;running-timer=0 -> error
		ldi		ZL,ontmrstart		;Basisadresse bergeben
		rcall	ldtmr				;runtmr mit on-wert laden
		breq	mainerr				;running-timer=0 -> error
		andi	status,0xD0			;alle Zyklus/running-Bits aus
		ori		status,0x64			;running-ON, Single Zyklus, first Single
		sei							;global Interrupt enable
		rjmp	mainok
cmdx:
		cpi		temp0,'x'			;Single ON - OFF?
		brne	cmdy
		cli							;global Interrupt disable
		ldi		ZL,ontmrstart		;Basisadresse bergeben
		rcall	chktmr				;runtmr mit on-wert laden
		breq	mainerr				;running-timer=0 -> error
		ldi		ZL,offtmrstart		;Basisadresse bergeben
		rcall	ldtmr				;runtmr mit off-wert laden
		breq	mainerr				;running-timer=0 -> error
		andi	status,0xD0			;alle Zyklus/running-Bits aus
		ori		status,0x44			;running-OFF, Single Zyklus, first Single
		sei							;global Interrupt enable
		rjmp	mainok
cmdy:
		cpi		temp0,'y'			;Zyklus OFF - ON...?
		brne	cmdz
		cli							;global Interrupt disable
		ldi		ZL,offtmrstart		;Basisadresse bergeben
		rcall	chktmr				;runtmr mit on-wert laden
		breq	mainerr				;running-timer=0 -> error
		ldi		ZL,ontmrstart		;Basisadresse bergeben
		rcall	ldtmr				;runtmr mit on-wert laden
		breq	mainerr				;running-timer=0 -> error
		andi	status,0xB0			;alle Zyklus/Single-Bits aus
		ori		status,0x28			;running-ON, Zyklus
		sei							;global Interrupt enable
		rjmp	mainok
cmdz:
		cpi		temp0,'z'			;Zyklus ON - OFF...?
		brne	mainerr
		cli							;global Interrupt disable
		ldi		ZL,ontmrstart		;Basisadresse bergeben
		rcall	chktmr				;runtmr mit on-wert laden
		breq	mainerr				;running-timer=0 -> error
		ldi		ZL,offtmrstart		;Basisadresse bergeben
		rcall	ldtmr				;runtmr mit off-wert laden
		breq	mainerr				;running-timer=0 -> error
		andi	status,0xB0			;alle Zyklus/Single-Bits aus
		ori		status,0x08			;running-OFF, Zyklus
		sei							;global Interrupt enable
		rjmp	mainok
mainerr:
		sei							;global Interrupt enable
		ldi		serbyte,'e'			;Error Befehl nicht erkannt
		rcall	putchar				;ausgeben
		ldi		serbyte,'r'			;Error Befehl nicht erkannt
		rcall	putchar				;ausgeben
		ldi		serbyte,'r'			;Error Befehl nicht erkannt
		rcall	putchar				;ausgeben
		ldi		serbyte,'o'			;Error Befehl nicht erkannt
		rcall	putchar				;ausgeben
		ldi		serbyte,'r'			;Error Befehl nicht erkannt
		rcall	putchar				;ausgeben
		rjmp	main

mainok:
		ldi		serbyte,'o'			;OK
		rcall	putchar				;ausgeben
		ldi		serbyte,'k'
		rcall	putchar
		rjmp	main


;***** translate byte stored in temp to two Hex-Digits temp1:temp2
bin2hex:
		mov		temp2,temp0			;wird noch gebraucht
		swap	temp0				;high Digit -> low
		andi	temp0,0x0F			;ausblenden
		cpi		temp0,10			;ist temp < 10?
		brmi	bhziff1				;ja, ziffer
		ldi		temp1,87			;Offset von 'a'-10
		add		temp0,temp1			;addieren
		rjmp	bhbst1				;Buchstabe ausgeben
bhziff1:
		ori		temp0,0x30			;Ascii Ziffer
bhbst1:
		mov		temp1,temp0			;fr Ausgabe laden
		andi	temp2,0x0F			;High Digit ausblenden
		cpi		temp2,10			;ist temp < 10?
		brmi	bhziff2				;ja, ziffer
		ldi		temp0,87			;Offset von 'a'-10
		add		temp2,temp0			;addieren
		ret							;fertig
bhziff2:
		ori		temp2,0x30			;Ascii Ziffer
		ret

;***** translate two Hex-Digits temp1:temp2 to byte in temp
hex2bin:
		clr		temp0				;fr spter
		cpi		temp1,0x3A			;Wert grer als Ziffer?
		brmi	hbziff1				;nein, Ziffer auswerten
		andi	temp1,0x07			;A..F/a..f = 0x61..0x66/0x41..0x46 := 3 Bits
		subi	temp1,-9			;ist wie Addition +9
hbziff1:
		andi	temp1,0x0F			;fr Ziffern muss nur high Digit wge
		or		temp0,temp1			;speichern fr Rckgabe
		swap	temp0				;an oberes Digit schieben
		cpi		temp2,0x3A			;Wert grer als Ziffer?
		brmi	hbziff2				;nein, Ziffer auswerten
		andi	temp2,0x07			;A..F/a..f = 0x61..0x66/0x41..0x46 := 3 Bits
		subi	temp2,-9			;ist wie Addition +9
hbziff2:
		andi	temp2,0x0F			;fr Ziffern muss nur high Digit wge
		or		temp0,temp2			;speichern fr Rckgabe
		ret


;***** transmit the byte stored in the "serbyte" register
putchar:sbis 	USR,UDRE			;Wait for data register empty
		rjmp	putchar				;weiter warten
		out		UDR,serbyte
		ret

;***** receive one byte and return it in the "serbyte" register
getchar:wdr
		sbis 	USR,RXC				;Wait for receiver
		rjmp	getchar				;weiter warten
		in		serbyte,UDR
		ret


.equ	ignoff		=-1-OVF0load	;Wert fr Zndung komplett aus
#define ignmin		20				;abhaengig von Bauelementestreuung
#define ignmax		140				;abhaengig von Bauelementestreuung
;#define	ignscale	1.8345			;Skalierungsfaktor fuer Zuendwert, 255 / (ignoff - ignmin)
#define	ignscale	2.125			;Skalierungsfaktor fuer Zuendwert, 255 / (ignmax - ignmin)

dimtable:
;0x00
		.dw		ignmin + int(0x00 / ignscale)
		.dw		ignmin + int(0x01 / ignscale)
		.dw		ignmin + int(0x02 / ignscale)
		.dw		ignmin + int(0x03 / ignscale)
		.dw		ignmin + int(0x04 / ignscale)
		.dw		ignmin + int(0x05 / ignscale)
		.dw		ignmin + int(0x06 / ignscale)
		.dw		ignmin + int(0x07 / ignscale)
		.dw		ignmin + int(0x08 / ignscale)
		.dw		ignmin + int(0x09 / ignscale)
		.dw		ignmin + int(0x0a / ignscale)
		.dw		ignmin + int(0x0b / ignscale)
		.dw		ignmin + int(0x0c / ignscale)
		.dw		ignmin + int(0x0d / ignscale)
		.dw		ignmin + int(0x0e / ignscale)
		.dw		ignmin + int(0x0f / ignscale)
;0x10
		.dw		ignmin + int(0x10 / ignscale)
		.dw		ignmin + int(0x11 / ignscale)
		.dw		ignmin + int(0x12 / ignscale)
		.dw		ignmin + int(0x13 / ignscale)
		.dw		ignmin + int(0x14 / ignscale)
		.dw		ignmin + int(0x15 / ignscale)
		.dw		ignmin + int(0x16 / ignscale)
		.dw		ignmin + int(0x17 / ignscale)
		.dw		ignmin + int(0x18 / ignscale)
		.dw		ignmin + int(0x19 / ignscale)
		.dw		ignmin + int(0x1a / ignscale)
		.dw		ignmin + int(0x1b / ignscale)
		.dw		ignmin + int(0x1c / ignscale)
		.dw		ignmin + int(0x1d / ignscale)
		.dw		ignmin + int(0x1e / ignscale)
		.dw		ignmin + int(0x1f / ignscale)
;0x20
		.dw		ignmin + int(0x20 / ignscale)
		.dw		ignmin + int(0x21 / ignscale)
		.dw		ignmin + int(0x22 / ignscale)
		.dw		ignmin + int(0x23 / ignscale)
		.dw		ignmin + int(0x24 / ignscale)
		.dw		ignmin + int(0x25 / ignscale)
		.dw		ignmin + int(0x26 / ignscale)
		.dw		ignmin + int(0x27 / ignscale)
		.dw		ignmin + int(0x28 / ignscale)
		.dw		ignmin + int(0x29 / ignscale)
		.dw		ignmin + int(0x2a / ignscale)
		.dw		ignmin + int(0x2b / ignscale)
		.dw		ignmin + int(0x2c / ignscale)
		.dw		ignmin + int(0x2d / ignscale)
		.dw		ignmin + int(0x2e / ignscale)
		.dw		ignmin + int(0x2f / ignscale)
;0x30
		.dw		ignmin + int(0x30 / ignscale)
		.dw		ignmin + int(0x31 / ignscale)
		.dw		ignmin + int(0x32 / ignscale)
		.dw		ignmin + int(0x33 / ignscale)
		.dw		ignmin + int(0x34 / ignscale)
		.dw		ignmin + int(0x35 / ignscale)
		.dw		ignmin + int(0x36 / ignscale)
		.dw		ignmin + int(0x37 / ignscale)
		.dw		ignmin + int(0x38 / ignscale)
		.dw		ignmin + int(0x39 / ignscale)
		.dw		ignmin + int(0x3a / ignscale)
		.dw		ignmin + int(0x3b / ignscale)
		.dw		ignmin + int(0x3c / ignscale)
		.dw		ignmin + int(0x3d / ignscale)
		.dw		ignmin + int(0x3e / ignscale)
		.dw		ignmin + int(0x3f / ignscale)
;0x40
		.dw		ignmin + int(0x40 / ignscale)
		.dw		ignmin + int(0x41 / ignscale)
		.dw		ignmin + int(0x42 / ignscale)
		.dw		ignmin + int(0x43 / ignscale)
		.dw		ignmin + int(0x44 / ignscale)
		.dw		ignmin + int(0x45 / ignscale)
		.dw		ignmin + int(0x46 / ignscale)
		.dw		ignmin + int(0x47 / ignscale)
		.dw		ignmin + int(0x48 / ignscale)
		.dw		ignmin + int(0x49 / ignscale)
		.dw		ignmin + int(0x4a / ignscale)
		.dw		ignmin + int(0x4b / ignscale)
		.dw		ignmin + int(0x4c / ignscale)
		.dw		ignmin + int(0x4d / ignscale)
		.dw		ignmin + int(0x4e / ignscale)
		.dw		ignmin + int(0x4f / ignscale)
;0x50
		.dw		ignmin + int(0x50 / ignscale)
		.dw		ignmin + int(0x51 / ignscale)
		.dw		ignmin + int(0x52 / ignscale)
		.dw		ignmin + int(0x53 / ignscale)
		.dw		ignmin + int(0x54 / ignscale)
		.dw		ignmin + int(0x55 / ignscale)
		.dw		ignmin + int(0x56 / ignscale)
		.dw		ignmin + int(0x57 / ignscale)
		.dw		ignmin + int(0x58 / ignscale)
		.dw		ignmin + int(0x59 / ignscale)
		.dw		ignmin + int(0x5a / ignscale)
		.dw		ignmin + int(0x5b / ignscale)
		.dw		ignmin + int(0x5c / ignscale)
		.dw		ignmin + int(0x5d / ignscale)
		.dw		ignmin + int(0x5e / ignscale)
		.dw		ignmin + int(0x5f / ignscale)
;0x60
		.dw		ignmin + int(0x60 / ignscale)
		.dw		ignmin + int(0x61 / ignscale)
		.dw		ignmin + int(0x62 / ignscale)
		.dw		ignmin + int(0x63 / ignscale)
		.dw		ignmin + int(0x64 / ignscale)
		.dw		ignmin + int(0x65 / ignscale)
		.dw		ignmin + int(0x66 / ignscale)
		.dw		ignmin + int(0x67 / ignscale)
		.dw		ignmin + int(0x68 / ignscale)
		.dw		ignmin + int(0x69 / ignscale)
		.dw		ignmin + int(0x6a / ignscale)
		.dw		ignmin + int(0x6b / ignscale)
		.dw		ignmin + int(0x6c / ignscale)
		.dw		ignmin + int(0x6d / ignscale)
		.dw		ignmin + int(0x6e / ignscale)
		.dw		ignmin + int(0x6f / ignscale)
;0x70
		.dw		ignmin + int(0x70 / ignscale)
		.dw		ignmin + int(0x71 / ignscale)
		.dw		ignmin + int(0x72 / ignscale)
		.dw		ignmin + int(0x73 / ignscale)
		.dw		ignmin + int(0x74 / ignscale)
		.dw		ignmin + int(0x75 / ignscale)
		.dw		ignmin + int(0x76 / ignscale)
		.dw		ignmin + int(0x77 / ignscale)
		.dw		ignmin + int(0x78 / ignscale)
		.dw		ignmin + int(0x79 / ignscale)
		.dw		ignmin + int(0x7a / ignscale)
		.dw		ignmin + int(0x7b / ignscale)
		.dw		ignmin + int(0x7c / ignscale)
		.dw		ignmin + int(0x7d / ignscale)
		.dw		ignmin + int(0x7e / ignscale)
		.dw		ignmin + int(0x7f / ignscale)
;0x80
		.dw		ignmin + int(0x80 / ignscale)
		.dw		ignmin + int(0x81 / ignscale)
		.dw		ignmin + int(0x82 / ignscale)
		.dw		ignmin + int(0x83 / ignscale)
		.dw		ignmin + int(0x84 / ignscale)
		.dw		ignmin + int(0x85 / ignscale)
		.dw		ignmin + int(0x86 / ignscale)
		.dw		ignmin + int(0x87 / ignscale)
		.dw		ignmin + int(0x88 / ignscale)
		.dw		ignmin + int(0x89 / ignscale)
		.dw		ignmin + int(0x8a / ignscale)
		.dw		ignmin + int(0x8b / ignscale)
		.dw		ignmin + int(0x8c / ignscale)
		.dw		ignmin + int(0x8d / ignscale)
		.dw		ignmin + int(0x8e / ignscale)
		.dw		ignmin + int(0x8f / ignscale)
;0x90
		.dw		ignmin + int(0x90 / ignscale)
		.dw		ignmin + int(0x91 / ignscale)
		.dw		ignmin + int(0x92 / ignscale)
		.dw		ignmin + int(0x93 / ignscale)
		.dw		ignmin + int(0x94 / ignscale)
		.dw		ignmin + int(0x95 / ignscale)
		.dw		ignmin + int(0x96 / ignscale)
		.dw		ignmin + int(0x97 / ignscale)
		.dw		ignmin + int(0x98 / ignscale)
		.dw		ignmin + int(0x99 / ignscale)
		.dw		ignmin + int(0x9a / ignscale)
		.dw		ignmin + int(0x9b / ignscale)
		.dw		ignmin + int(0x9c / ignscale)
		.dw		ignmin + int(0x9d / ignscale)
		.dw		ignmin + int(0x9e / ignscale)
		.dw		ignmin + int(0x9f / ignscale)
;0xa0
		.dw		ignmin + int(0xa0 / ignscale)
		.dw		ignmin + int(0xa1 / ignscale)
		.dw		ignmin + int(0xa2 / ignscale)
		.dw		ignmin + int(0xa3 / ignscale)
		.dw		ignmin + int(0xa4 / ignscale)
		.dw		ignmin + int(0xa5 / ignscale)
		.dw		ignmin + int(0xa6 / ignscale)
		.dw		ignmin + int(0xa7 / ignscale)
		.dw		ignmin + int(0xa8 / ignscale)
		.dw		ignmin + int(0xa9 / ignscale)
		.dw		ignmin + int(0xaa / ignscale)
		.dw		ignmin + int(0xab / ignscale)
		.dw		ignmin + int(0xac / ignscale)
		.dw		ignmin + int(0xad / ignscale)
		.dw		ignmin + int(0xae / ignscale)
		.dw		ignmin + int(0xaf / ignscale)
;0xb0
		.dw		ignmin + int(0xb0 / ignscale)
		.dw		ignmin + int(0xb1 / ignscale)
		.dw		ignmin + int(0xb2 / ignscale)
		.dw		ignmin + int(0xb3 / ignscale)
		.dw		ignmin + int(0xb4 / ignscale)
		.dw		ignmin + int(0xb5 / ignscale)
		.dw		ignmin + int(0xb6 / ignscale)
		.dw		ignmin + int(0xb7 / ignscale)
		.dw		ignmin + int(0xb8 / ignscale)
		.dw		ignmin + int(0xb9 / ignscale)
		.dw		ignmin + int(0xba / ignscale)
		.dw		ignmin + int(0xbb / ignscale)
		.dw		ignmin + int(0xbc / ignscale)
		.dw		ignmin + int(0xbd / ignscale)
		.dw		ignmin + int(0xbe / ignscale)
		.dw		ignmin + int(0xbf / ignscale)
;0xc0
		.dw		ignmin + int(0xc0 / ignscale)
		.dw		ignmin + int(0xc1 / ignscale)
		.dw		ignmin + int(0xc2 / ignscale)
		.dw		ignmin + int(0xc3 / ignscale)
		.dw		ignmin + int(0xc4 / ignscale)
		.dw		ignmin + int(0xc5 / ignscale)
		.dw		ignmin + int(0xc6 / ignscale)
		.dw		ignmin + int(0xc7 / ignscale)
		.dw		ignmin + int(0xc8 / ignscale)
		.dw		ignmin + int(0xc9 / ignscale)
		.dw		ignmin + int(0xca / ignscale)
		.dw		ignmin + int(0xcb / ignscale)
		.dw		ignmin + int(0xcc / ignscale)
		.dw		ignmin + int(0xcd / ignscale)
		.dw		ignmin + int(0xce / ignscale)
		.dw		ignmin + int(0xcf / ignscale)
;0xd0
		.dw		ignmin + int(0xd0 / ignscale)
		.dw		ignmin + int(0xd1 / ignscale)
		.dw		ignmin + int(0xd2 / ignscale)
		.dw		ignmin + int(0xd3 / ignscale)
		.dw		ignmin + int(0xd4 / ignscale)
		.dw		ignmin + int(0xd5 / ignscale)
		.dw		ignmin + int(0xd6 / ignscale)
		.dw		ignmin + int(0xd7 / ignscale)
		.dw		ignmin + int(0xd8 / ignscale)
		.dw		ignmin + int(0xd9 / ignscale)
		.dw		ignmin + int(0xda / ignscale)
		.dw		ignmin + int(0xdb / ignscale)
		.dw		ignmin + int(0xdc / ignscale)
		.dw		ignmin + int(0xdd / ignscale)
		.dw		ignmin + int(0xde / ignscale)
		.dw		ignmin + int(0xdf / ignscale)
;0xe0
		.dw		ignmin + int(0xe0 / ignscale)
		.dw		ignmin + int(0xe1 / ignscale)
		.dw		ignmin + int(0xe2 / ignscale)
		.dw		ignmin + int(0xe3 / ignscale)
		.dw		ignmin + int(0xe4 / ignscale)
		.dw		ignmin + int(0xe5 / ignscale)
		.dw		ignmin + int(0xe6 / ignscale)
		.dw		ignmin + int(0xe7 / ignscale)
		.dw		ignmin + int(0xe8 / ignscale)
		.dw		ignmin + int(0xe9 / ignscale)
		.dw		ignmin + int(0xea / ignscale)
		.dw		ignmin + int(0xeb / ignscale)
		.dw		ignmin + int(0xec / ignscale)
		.dw		ignmin + int(0xed / ignscale)
		.dw		ignmin + int(0xee / ignscale)
		.dw		ignmin + int(0xef / ignscale)
;0xf0
		.dw		ignmin + int(0xf0 / ignscale)
		.dw		ignmin + int(0xf1 / ignscale)
		.dw		ignmin + int(0xf2 / ignscale)
		.dw		ignmin + int(0xf3 / ignscale)
		.dw		ignmin + int(0xf4 / ignscale)
		.dw		ignmin + int(0xf5 / ignscale)
		.dw		ignmin + int(0xf6 / ignscale)
		.dw		ignmin + int(0xf7 / ignscale)
		.dw		ignmin + int(0xf8 / ignscale)
		.dw		ignmin + int(0xf9 / ignscale)
		.dw		ignmin + int(0xfa / ignscale)
		.dw		ignmin + int(0xfb / ignscale)
		.dw		ignmin + int(0xfc / ignscale)
		.dw		ignmin + int(0xfd / ignscale)
		.dw		ignmin + int(0xfe / ignscale)
		.dw		ignmin + int(0xff / ignscale)
